#include <Arduino.h>
#include "application.h"
#include "synth_string.h"

//- - - - - - - - - - - - - -

void synth_instrument_string::startInstrument( ui_controls_struct *ui, int key ) // float frequency )
{
  // calc frequency for note
  float frequency = NOTES[ (NUM_SCALES-1) - instrument[ui->wave_knob].notes_scale ].frequency[ key ];
  // pitch is 0..4
  //   but only 1..4 is valid
  byte oct = ui->band_knob + 1;
   if (oct > 4) oct = 4;
  for (int m=0; m <= oct; m++) frequency *= 2;
  
  // lowest frequency is 100 Hz @ 22 KHz
  bufferLen = (AUDIO_SAMPLE_RATE / frequency) + 0.5f;
  if (bufferLen > string_bufsiz) bufferLen = string_bufsiz;
  bufferIndex = 0;

  uint32_t lo = seed;
  for (int i=0; i < bufferLen; i++) 
  { //lo = pseudorand(lo);
    uint32_t hi;
    // hi = multiply_16bx16t(16807, lo); // 16807 * (lo >> 16)
    hi = 16807 * (lo >> 16);
    lo = 16807 * (lo & 0xFFFF);
    lo += (hi & 0x7FFF) << 16;
    lo += hi >> 15;
    lo = (lo & 0x7FFFFFFF) + (lo >> 31);
    buffer[i] = lo; //signed_multiply_32x16b(magnitude, lo);
  }
  seed = lo;

  updateInstrument( ui );  // set proximity effect
}

//- - - - - - - - - - - - - -

void synth_instrument_string::stopInstrument()
{
  bufferLen = 0;
}

//- - - - - - - - - - - - - -

void synth_instrument_string::updateInstrument( ui_controls_struct *ui )
{
  // dial effect
  decay = BASE_DECAY - ui->effect_knob*250;
  // proximity effect
  if (ui->proximity_enabled)
    if (ui->proximity > 5) decay -= ui->proximity * (NBR_STRING_EFFECTS+1-ui->effect_knob)*2;  
}

//- - - - - - - - - - - - - -

byte synth_instrument_string::runInstrument( int32_t *buf )
{
  byte running = 0;
  if (bufferLen)
  {
    for (int i=0; i < SOUND_BLOCK_SIZE; i++) 
    {
      int16_t in  = buffer[bufferIndex];
      int16_t out = (in * decay + prior * decay) >> 16;
      *(buf++) += buffer[bufferIndex] = out;
      prior = in;
      if (++bufferIndex >= bufferLen) bufferIndex = 0;
      if (out != -200) running = 1;
    }
  }
  return running;
}


