// STM32F103 capsense

#include "Arduino.h"
#include "touchSensor.h"

//- - - - - - - - - - - - - -

void touchSensor::begin( byte analog_pin, uint16_t samples ) 
{
  pin       = analog_pin;
  reference = readRaw( samples );
}

//- - - - - - - - - - - - - -
// read capsense ADC value

uint16_t touchSensor::readRaw( uint16_t samples ) 
{
    int32_t total = 0;
    for (uint16_t i=0; i < samples; i++) 
    {
      pinMode(pin, INPUT_PULLUP);
      delayMicroseconds(20);
      adc_read(ADC1, 15); // ground
      pinMode(pin, INPUT_ANALOG);
      total += analogRead(pin);
    }
    return total / samples;
}

//- - - - - - - - - - - - - -
// is sensor touched now

boolean touchSensor::isTouched( uint16_t samples ) 
{
  return (readRaw( samples ) > reference + TOUCH_THRESHOLD_ADC );
}

//- - - - - - - - - - - - - -
// debounced touch event
// poll from loop()

int touchSensor::touchEvent( uint16_t samples ) 
{
  boolean t = isTouched();

  if (t)
  { tm = millis(); // refresh debounce timer
    if (!state)
    { state = true;
      return 1; // touch event detected
    }
  }
  else // not touched
    if (millis() - tm > TOUCH_DEBOUNCE_MS)
      if (state) { state = false;  return -1; } // release event detected
  
  return 0;
}

